@@ -8,6 +8,13 @@ from types import SimpleNamespace as structdata
 
 import spynnaker8 as sim
 
+import quantities as pq
+# for plotting
+import matplotlib.pyplot as plt
+import matplotlib.patches as mpatches
+from matplotlib import gridspec
+from matplotlib import cm
+
 from .nealassoc.readInheritanceFile import InheritanceReaderClass
 from .nealassoc.readUnitFile import UnitReaderClass
 #from readAssocFile import AssocReaderClass
@@ -179,6 +186,19 @@ class NEAL3Way(object):
         #
         neal.nealApplyProjections()
         sim.run(self.simTime)
+        [neo_base, neo_property, neo_relation] = self.__getdata()
+        self.results = { "base": self.__split_spiketrains("basedata", neo_base, turnon),
+                         "property": self.__split_spiketrains("propdata", neo_property, turnon),
+                         "relation": self.__split_spiketrains("reldata", neo_relation, turnon) }
+        #sim.reset()
+        sim.end()
+        
+    def get_results(self):
+        """Returns a dictionary with keys "base", "property", and "relation" whose values are dictionaries.
+        A value dictionary is such that the keys are the names of the cell assemblies (i.e. "units" in respective structured data)
+        and values are the respective spike trains (i.e. from all cell units in an assembly).
+        """
+        return self.results
         
     def __create_datastructures(self, bases, associate):
         """."""
@@ -213,13 +233,17 @@ class NEAL3Way(object):
         self.neural3assoc_topology.addAssociations( self.assocdata )
         
     def __choose_applicable_test(self, turnon):
-        """."""
+        """If ``turnon = "all"`` then all the cell assemblies for each subject (base), object (property) and relation (predicate) is activated.
+        On the other hand, if ``turnon = [<subject-name>, <predicate-name>, <object-name>]`` only cell assemblies for these are trigerred."""
         if turnon=="all":
             self.simTime = self.neural3assoc_topology.createUnitTests() + 100
         else:
-            baseNum = self.basedata.getUnitNumber( turnon[0] )
-            probNum = self.propdata.getUnitNumber( turnon[1] )
-            relNum = self.reldata.getUnitNumber( turnon[2] )
+            baseNum = self.basedata.getUnitNumber( turnon[0] ) # base or subject
+            probNum = self.propdata.getUnitNumber( turnon[1] ) # property or object
+            try:
+                relNum = self.reldata.getUnitNumber( turnon[2] )# relation or predicate
+            except:
+                relNum = -1 # cell assembly for relation is not activated
             self.neural3assoc_topology.createTwoPrimeTest( baseNum, probNum, relNum )
             self.simTime = 200.0
     
@@ -230,9 +254,32 @@ class NEAL3Way(object):
         neo_property = self.neural3assoc_topology.propertyCells.get_data(variables=["spikes"])
         neo_relation = self.neural3assoc_topology.relationCells.get_data(variables=["spikes"])
         return [neo_base, neo_property, neo_relation]
+        
+    # Private function for return overall spike train 
+    def __get_overallspikes(self, dataname, neo_data, turnon):
+        """Returns `Neo SpikeTrain <https://neo.readthedocs.io/en/stable/api_reference.html#neo.core.SpikeTrain>`_.
+        All of it if the data structure is basedata other wise the they are shifted accordingly for propdata and reldata.
+        """
+        if turnon=="all":
+            if dataname=="basedata":
+                return neo_data.segments[0].spiketrains
+            else:
+                if dataname=="propdata":
+                    data = getattr(self, "basedata")
+                    tstart = data.numberUnits * 100 # this 100 is the same value as in :py:meth:`.__choose_applicable_test`
+                else:
+                    data1 = getattr(self, "basedata")
+                    data2 = getattr(self, "propdata")
+                    tstart = (data1.numberUnits * 100) + (data2.numberUnits * 100)
+                spktrains_all = []
+                for spktrain in neo_data.segments[0].spiketrains:
+                    spktrains_all.append( spktrain - tstart*pq.ms )
+                return spktrains_all
+        else:
+            return neo_data.segments[0].spiketrains
     
     # Private function for segregating respective spike trains for each cell assembly within a Neo object.
-    def __split_spiketrains(self, dataname, neo_data):
+    def __split_spiketrains(self, dataname, neo_data, turnon):
         """Given the Neo object of a given data name ("basedata" or "propdata" or "reldata", see :py:meth:`.__create_datastructures`),
         this function returns a dictionary with "units" in the structured data as keys and its value the respective spike trains (all cell units in an assembly, i.e. the "unit").
         """
@@ -241,7 +288,8 @@ class NEAL3Way(object):
         spkindices = lambda n : (0,ca_size) if (n==0) else ( (n*ca_size), (n*ca_size)+ca_size )
         #
         data = getattr(self, dataname) # "basedata" or "propdata" or "reldata"
-        overallspikes = neo_data.segments[0].spiketrains
+        #overallspikes = neo_data.segments[0].spiketrains
+        overallspikes = self.__get_overallspikes(dataname, neo_data, turnon)
         #
         parsed_spiketrains = {"all": overallspikes} # this will be the returned value
         for unit in data.units:
@@ -249,13 +297,107 @@ class NEAL3Way(object):
             indx = spkindices(n)
             parsed_spiketrains.update( {unit: overallspikes[ indx[0] : indx[-1] ] } )
         return parsed_spiketrains
-            
-    def get_results(self):
-        """Returns a dictionary with keys "base", "property", and "relation" whose values are dictionaries.
-        A value dictionary is such that the keys are the names of the cell assemblies (i.e. "units" in respective structured data)
-        and values are the respective spike trains (i.e. from all cell units in an assembly).
+    
+    # Plotting functions
+    # Private function to get first key for self.results dictionary
+    def __get_resultskey(self, dataname):
+        """Returns key string for ``self.results``. This is because the attribute names for the data structures are a shortened
+        version of the keys in ``self.results`` with the exception of ``self.basedata``."""
+        if dataname=="reldata":
+            return "relation" # corresponding key in self.results is "relation"
+        elif dataname=="propdata":
+            return "property" # corresponding key in self.results is "property"
+        else:
+            return dataname.strip("data") # basedata -> base
+    
+    # Private function for each subplot in :py:meth:`.plot_all`
+    def __subplot_all(self, dataname, subplotobject, clrs):
+        """For a given ``subplotobject`` this function returns matplotlib's `eventplot <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.eventplot.html>`_
+        for spiketrains from all the cell units of all the cell assemblies for a given data structure (assigned by the ``dataname`` argument)."""
+        legpatches = []
+        data = getattr(self, dataname) # "basedata" or "propdata" or "reldata"
+        for unit in data.units:
+            i = data.getUnitNumber(unit)
+            subplotobject.eventplot( self.results[ self.__get_resultskey(dataname) ][ unit ],
+                                     color = clrs(1.0 - (i*0.1) ) )
+            legpatches.append( mpatches.Patch(color=clrs( 1.0-(i*0.1) ), label=unit) )
+        subplotobject.legend( handles=legpatches, shadow=True )
+    
+    def plot_all(self):
+        """Plot of three subplots such that each subplot (invoking :py:meth:`.__subplot_all`) is the matplotlib's `eventplot <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.eventplot.html>`_
+        of `SpikeTrain <https://neo.readthedocs.io/en/stable/api_reference.html#neo.core.SpikeTrain>`_ such that
+        
+        +---------+------------------------------------------------------------+-----------+
+        | subplot | spike trains for all cell units in all cell assemblies for | color map |
+        +=========+============================================================+===========+
+        | top     | subject (a.k.a base in ``basedata``)                       | Reds      |
+        +---------+------------------------------------------------------------+-----------+
+        | middle  | predicate (a.k.a relation in ``reldata``)                  | Greens    |
+        +---------+------------------------------------------------------------+-----------+
+        | bottom  | object (a.k.a property in ``propdata``)                    | Blues     |
+        +---------+------------------------------------------------------------+-----------+
+        
         """
-        [neo_base, neo_property, neo_relation] = self.__getdata()
-        return { "base": self.__split_spiketrains("basedata", neo_base),
-                 "property": self.__split_spiketrains("propdata", neo_property),
-                 "relation": self.__split_spiketrains("reldata", neo_relation) }
+        fig, ((sp1),
+              (sp2),
+              (sp3)) = plt.subplots(3,1, sharex=True)
+        # Red color for subject, i.e. base Therefore its spectrum is used here
+        clrs = cm.get_cmap('Reds', 12)
+        self.__subplot_all("basedata", sp1, clrs)
+        #
+        # Green color for predicate, i.e. relation
+        clrs = cm.get_cmap('Greens', 12)
+        self.__subplot_all("reldata", sp2, clrs)
+        #
+        # Blue color for object, i.e. property
+        clrs = cm.get_cmap('Blues', 12)
+        self.__subplot_all("propdata", sp3, clrs)
+        #
+        sp1.title.set_text("Subject (base)")
+        sp2.title.set_text("Predicate (relation)")
+        sp3.title.set_text("Object (property)")
+        #
+        sp1.set(ylabel="cell units\nper CA")
+        sp2.set(ylabel="cell units\nper CA")
+        sp3.set(ylabel="cell units\nper CA")
+        sp3.set(xlabel="time (ms)")
+        #
+        plt.subplots_adjust( hspace=0.5 ) # spacing for the each subplot title
+        plt.show()
+        
+    def plot_specific(self, basename=None, relname=None, propname=None):
+        """Plot of three subplots such that each is the matplotlib's `eventplot <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.eventplot.html>`_
+        of `SpikeTrain <https://neo.readthedocs.io/en/stable/api_reference.html#neo.core.SpikeTrain>`_ of all the cell units in cell assembly for given
+        
+        * ``basename`` in top subplot for subject, color is red
+        * ``relname`` in middle subplot for predicate, color is green
+        * ``propname`` in bottom subplot for object, color is blue
+        
+        """
+        fig, ((sp1),
+              (sp2),
+              (sp3)) = plt.subplots(3,1, sharex=True)
+        #
+        sp1.eventplot( self.results["base"][basename], color="red" ) # animal
+        z_patch = mpatches.Patch( color="red", label=basename )
+        sp1.legend( handles=[z_patch], shadow=True )
+        #
+        sp2.eventplot( self.results["relation"][relname], color="green" ) # eats
+        z_patch = mpatches.Patch( color="green", label=relname )
+        sp2.legend( handles=[z_patch], shadow=True )
+        #
+        sp3.eventplot( self.results["property"][propname], color="blue" ) # food
+        z_patch = mpatches.Patch( color="blue", label=propname )
+        sp3.legend( handles=[z_patch], shadow=True )
+        #
+        sp1.title.set_text('Subject (base)')
+        sp2.title.set_text('Predicate (relation)')
+        sp3.title.set_text('Object (property)')
+        #
+        sp1.set(ylabel="cell units\nper CA")
+        sp2.set(ylabel="cell units\nper CA")
+        sp3.set(ylabel="cell units\nper CA")
+        sp3.set(xlabel="time (ms)")
+        #
+        plt.subplots_adjust( hspace=.5 ) # spacing for the each subplot title
+        plt.show()
\ No newline at end of file
